package ui.internal;

import ui.Action;

public class Maze {
	
	private MazeData mazeData;
	
	public MazeData getMazeData() {
		return mazeData;
	}
	public void setMazeData(MazeData mazeData) {
		this.mazeData = mazeData;
	}
	
	public int getWidth() {
		int width = mazeData.mazeRewards.length;
		return width;
	}
	
	public int getHeight() {
		int height= mazeData.mazeRewards[0].length;
		return height;
	}  
	
	public void init(int mazeWidth, int mazeHeight, double defaultReward) {
		mazeData = new MazeData();
		
		mazeData.mazeRewards = new double[mazeWidth][mazeHeight];
		mazeData.absorbingGoalStates = new boolean[mazeWidth][mazeHeight];
		mazeData.teleportStates = new boolean[mazeWidth][mazeHeight];
		
		mazeData.verticalWalls = new boolean[mazeWidth+1][mazeHeight];
		mazeData.horizontalWalls = new boolean[mazeWidth][mazeHeight+1];
		
		
		// set default reward
		for (int x = 0; x < mazeWidth; x++) {
			for (int y = 0; y < mazeHeight; y++) {
				mazeData.mazeRewards[x][y] = defaultReward;
			}
		} 
		// reset absorbing goal states
		for (int x = 0; x < mazeWidth; x++) {
			for (int y = 0; y < mazeHeight; y++) {
				mazeData.absorbingGoalStates[x][y] = false;
				mazeData.teleportStates[x][y] = false;
			}
		} 
		
		// reset vertical walls
		
		for (int x = 0; x < mazeWidth+1; x++) {
			for (int y = 0; y < mazeHeight; y++) {
				mazeData.verticalWalls[x][y] = false;
			}
		} 
		
		// reset horizontal walls
		
		for (int x = 0; x < mazeWidth; x++) {
			for (int y = 0; y < mazeHeight+1; y++) {
				mazeData.horizontalWalls[x][y] = false;
			}
		} 
		
		// init boundary walls
		
		for (int x = 0; x < mazeWidth; x++) {
			mazeData.horizontalWalls[x][0] = true;
		}
		
		for (int x = 0; x < mazeWidth; x++) {
			mazeData.horizontalWalls[x][mazeHeight] = true;
		}
		
		for (int y = 0; y < mazeHeight; y++) {
			mazeData.verticalWalls[0][y] = true;
		}
		
		for (int y = 0; y < mazeHeight; y++) {
			mazeData.verticalWalls[mazeWidth][y] = true;
		}
	}
	
	
	
	private void setVerticalWall(int newX, int newY, boolean wall) {
		try {
			mazeData.verticalWalls[newX][newY] = wall;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private void setHorizontalWall(int newX, int newY, boolean wall) {
		try {
			mazeData.horizontalWalls[newX][newY] = wall;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public double getReward(int x, int y) {
	 	return mazeData.mazeRewards[x][y];
	}
	
	public void setReward(int x, int y, double reward) {
	 	mazeData.mazeRewards[x][y] = reward;
	}
	
	public void setWall(int x, int y, Action action, boolean wall) {
		
		if (Action.GO_WEST.equals(action)) {
		 	int newX = x; 
		 	int newY = y;
			setVerticalWall(newX, newY, wall);
		}

		if (Action.GO_NORTH.equals(action)) {
		 	int newX = x; 
		 	int newY = y;
			setHorizontalWall(newX, newY, wall);
		}
		
		
		if (Action.GO_SOUTH.equals(action)) {
		 	int newX = x; 
		 	int newY = y+1;
			setHorizontalWall(newX, newY, wall);
		}
		
		if (Action.GO_EAST.equals(action)) {
		 	int newX = x+1; 
		 	int newY = y;
		 	
			setVerticalWall(newX, newY, wall);
		}
	}
	
	
	public boolean isAbsorbingGoal(int x, int y) {
		return mazeData.absorbingGoalStates[x][y];
	}
	
	public void setAbsorbingGoal(int x, int y, boolean absorbingGoal) {
		mazeData.absorbingGoalStates[x][y] = absorbingGoal;
	}
	

	
	
	public boolean isTeleportState(int x, int y) {
		return mazeData.teleportStates[x][y];
	}
	
	public void setTeleportState(int x, int y, boolean teleportState) {
		mazeData.teleportStates[x][y] = teleportState;
	}
	
	
	public boolean isBorder(int x, int y, Action action) {
		
		if ((x == 0) && (Action.GO_WEST.equals(action))) {
			return true;
		}
		
		if ((x == -1) && (Action.GO_EAST.equals(action))) {
			return true;
		}
		
		
		if ((y == 0) && (Action.GO_NORTH.equals(action))) {
			return true;
		}
		
		if ((y == -1) && (Action.GO_SOUTH.equals(action))) {
			return true;
		}
		
		if ((x == getWidth()-1) && (Action.GO_EAST.equals(action))) {
			return true;
		}
		if ((x == getWidth()) && (Action.GO_WEST.equals(action))) {
			return true;
		}
		
		if ((y == getHeight()-1) &&  (Action.GO_SOUTH.equals(action))) {
			return true;
		}
		if ((y == getHeight()) &&  (Action.GO_NORTH.equals(action))) {
			return true;
		}
		
		return false;
	}
	
	public boolean isWall(int x, int y, Action action) {

		if (Action.GO_WEST.equals(action)) {
			int newX = x;
			int newY = y;
			return mazeData.verticalWalls[newX][newY];
		}

		if (Action.GO_NORTH.equals(action)) {
			int newX = x;
			int newY = y;
			return mazeData.horizontalWalls[newX][newY];
		}

		if (Action.GO_SOUTH.equals(action)) {
			int newX = x;
			int newY = y + 1;
			return mazeData.horizontalWalls[newX][newY];
		}

		if (Action.GO_EAST.equals(action)) {
			int newX = x + 1;
			int newY = y;
			return mazeData.verticalWalls[newX][newY];
		}

		throw new RuntimeException("bad action used");
	}
	
	
}
