package ui.internal;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

public class NontTerminalStateRewardSetter {

	private static final String MAZE_DIR="./mazes/";
	private static final double NON_TERMINAL_REWARD = -10;
	
	
	public static void main(String[] args) {
		File dir = new File(MAZE_DIR);
		
		String[] mazeFileNames = dir.list();
		
		for (String mazeFileName : mazeFileNames) {
			updateMaze(MAZE_DIR+mazeFileName, NON_TERMINAL_REWARD);
		}
	}
	
	private static void updateMaze(String fileName, double newNonTerminalRward) {
		MazeData mazeData = loadMazeData(fileName);
		
		Maze maze = new Maze();
		maze.setMazeData(mazeData);
		
		int width = maze.getWidth();
		int height = maze.getHeight();
		
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				if (!maze.isAbsorbingGoal(x, y) && (!maze.isTeleportState(x, y))) {
					maze.setReward(x, y, newNonTerminalRward);
				}
			}
		}
		
		MazeData updatedMaze = maze.getMazeData();
		
		saveMazeData(fileName, updatedMaze);
		
	}
	
	
	
	private static MazeData loadMazeData(String fileName) {
		MazeData mazeData = null;
		try {
			FileInputStream fis = new FileInputStream(fileName);
			GZIPInputStream gzis = new GZIPInputStream(fis);
			ObjectInputStream in = new ObjectInputStream(gzis);

			mazeData = ((MazeData) in.readObject());

			in.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return mazeData;
	}

	private static void saveMazeData(String fileName, MazeData updatedMaze) {
		try {
			File file = new File(fileName);
			FileOutputStream fos = new FileOutputStream(file);
			GZIPOutputStream gzos = new GZIPOutputStream(fos);
			ObjectOutputStream out = new ObjectOutputStream(gzos);

			out.writeObject(updatedMaze);

			out.flush();
			out.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
}
