package ui.internal;


/**
 * @author Petr
 *
 * Class for representing states. State can be regular - representing grid cell 
 * including position of the cell in the grid.
 * Regular states are 'normal state', 'teleport state' and 'absorbing state'.
 * 
 * Next state after teleport state is randomly selected regular state (including this teleport state) independently on
 * performed action.
 * 
 * Next state after 'absorbing state' is irregular 'dummy state' independently on performed action.
 * 
 * Next state after 'dummy state' is 'dummy state' independently on performed action. 
 * 
 * 
 * State can be a teleport state. It means, that next state after teleport one is 
 *
 *Absorbing states
 * There is one dummy state. It is impossible to leave the dummy state.
 * 
 * Each state has a reward. Dummy state has zero reward.
 *
 */
public class State implements StateInterface {
	
	private boolean isAbsorbing = false;
	private boolean isTeleport = false;
	private boolean isDummy = false;
	
	
	private int x = 0;
	private int y = 0;
	
	private double reward = 0; // def falue for dummy state;
	
	// constructor for dummy state
	
	public State() {
		isDummy = true;
	}
	
	public State(int x, int y, double reward, boolean isAbsorbing, boolean isTeleport) {
		this.x = x;
		this.y = y;
		this.reward = reward;
		this.isAbsorbing = isAbsorbing;
		this.isTeleport = isTeleport;
	}

	@Override
	public int getX() {
		return x;
	}

	@Override
	public int getY() {
		return y;
	}
	
	@Override
	public boolean isAbsorbingState() {
		return isAbsorbing;
	}

	@Override
	public boolean isTeleportState() {
		return isTeleport; 
	}

	@Override
	public double getReward() {
		return reward;
	}

	@Override
	public boolean isDummyState() {
		return isDummy;
	}
	

}
